#=====================================================================
# Model ULSP bez zmiennych zapasw
#=====================================================================

param Model symbolic := "ULSP_bez_zapasw";
param Dane symbolic;
param Wyniki symbolic := Model&"_"&Dane&".csv";
param Podsumowanie symbolic := "Podsumowanie.csv";

#=====================================================================
param L_tygodni > 0, integer;
set Tygodnie := 1..L_tygodni;

#=====================================================================
param Czas_pracy_na_tydzien > 0;

param Gotowosc_pocz binary;

param Czas_wykonywania >= 0;
param Czas_przezbr >= 0;
param Koszt_przezbr >= 0;
param Jedn_koszt_zapasow >= 0;

param Popyt {Tygodnie} >= 0;
param Pozostaly_Popyt {t in Tygodnie} :=
	sum {s in Tygodnie: t <= s} Popyt[t];

#=====================================================================
var Gotowosc {Tygodnie} binary;
var Produkcja {Tygodnie} >= 0;
var Zapas {Tygodnie union {0}} >= 0;

var Calk_koszt_zapasow >= 0;
var Calk_koszt_przezbr >= 0;

#=====================================================================
minimize FC_1_6_a_Koszty_calk:
	Calk_koszt_przezbr + Calk_koszt_zapasow;

subject to

	FC_1_6_a_Calk_koszt_przezbr:
		Calk_koszt_przezbr =
			sum {t in Tygodnie}
				Koszt_przezbr * Gotowosc[t];

	FC_1_6_a_Calk_koszt_zapasow:
		Calk_koszt_zapasow =
			sum {t in Tygodnie}
				Jedn_koszt_zapasow * sum {s in Tygodnie: s <= t}
						(Produkcja[s] - Popyt[s]);

	#--------------------------------------------------------------------------------------------------
	Ogr_1_6_b_Ogr_Bilans_zapasow {t in Tygodnie}:
		sum {s in Tygodnie: s <= t}
			(Produkcja[s] - Popyt[s]) >= 0;

	Ogr_1_6_c_Ogr_Produkcja_gdy_Gotowosc {t in Tygodnie}:
		Produkcja[t]  <= Pozostaly_Popyt[t] * Gotowosc[t];

#=====================================================================
	display Pozostaly_Popyt;

	solve;

	printf "Zmienna;Tydzie;Warto" > Wyniki;

	printf "\n%s;;%s", 'Model',Model >> Wyniki;
	printf "\n%s;;%s", 'Dane',Dane >> Wyniki;
	printf "\n%s;;%s", 'Data', time2str(gmtime(),'%F') >> Wyniki;
	printf "\n%s;;%s", 'Godzina', time2str(gmtime(),'%H:%M') >> Wyniki;
	printf "\n%s;;%d", 'L_tygodni', L_tygodni >> Wyniki;
	printf "\n%s;;%d", 'Czas_pracy', Czas_pracy_na_tydzien >> Wyniki;

	printf "\n%s;;%3.1f", 'Calk_koszt_zapasow', Calk_koszt_zapasow >> Wyniki;
	printf "\n%s;;%3.1f", 'Calk_koszt_przezbr', Calk_koszt_przezbr >> Wyniki;

	printf "\n%s;;%3.1f", 'Czas_wykonywania', Czas_wykonywania >> Wyniki;
	printf "\n%s;;%3.1f", 'Koszt_przezbr', Koszt_przezbr >> Wyniki;
	printf "\n%s;;%3.1f", 'Czas_przezbr', Czas_przezbr >> Wyniki;
	printf "\n%s;;%3.1f", 'Jedn_koszt_zapasow', Jedn_koszt_zapasow >> Wyniki;

	for {t in Tygodnie} {
		printf "\n%s;%d;%3.1f", 'Popyt', t, Popyt[t] >> Wyniki;
		printf "\n%s;%d;%3.1f", 'Produkcja', t, Produkcja[t] >> Wyniki;
		printf "\n%s;%d;%d", 'Gotowosc', t, Gotowosc[t] >> Wyniki;
	}

	for {t in Tygodnie union {0}}
		printf "\n%s;%d;%3.1f", 'Zapas', t,
			sum {s in Tygodnie: s <= t}
				(Produkcja[s] - Popyt[s]) >> Wyniki;

	#--------------------------------------------------------------------------------------------------
	printf "%s;%s;%3.1f;%3.1f;%3.1f\n", Model, Dane,
		Calk_koszt_zapasow , Calk_koszt_przezbr,
		sum {t in Tygodnie} Gotowosc[t]
		>> Podsumowanie ;

#=====================================================================
end;
