#=====================================================================
# Model ULSP
#=====================================================================

param Model symbolic := "ULSP_z_odciciami";
param Dane symbolic;
param Wyniki symbolic := Model&"_"&Dane&".csv";
param Podsumowanie symbolic := "Podsumowanie.csv";

#=====================================================================
param L_tygodni > 0, integer;
set Tygodnie := 1..L_tygodni;

#=====================================================================
param Czas_pracy_na_tydzien > 0;

param Gotowosc_pocz binary;

param Czas_wykonywania >= 0;
param Czas_przezbr >= 0;
param Koszt_przezbr >= 0;
param Jedn_koszt_zapasow >= 0;

param Popyt {Tygodnie} >= 0;
param Pozostaly_popyt {t in Tygodnie} :=
	sum {s in Tygodnie: t <= s} Popyt[t];

#=====================================================================
var Gotowosc {Tygodnie} binary;
var Produkcja {Tygodnie} >= 0;
var Zapas {Tygodnie union {0}} >= 0;

var Calk_koszt_zapasow >= 0;
var Calk_koszt_przezbr >= 0;

#=====================================================================
minimize FC_1_3_a_Koszty_calk:
	Calk_koszt_przezbr + Calk_koszt_zapasow;

subject to

	FC_1_3_a_Calk_koszt_przezbr:
		Calk_koszt_przezbr =
			sum {t in Tygodnie}
				Koszt_przezbr * Gotowosc[t];

	FC_1_3_a_Calk_koszt_zapasow:
		Calk_koszt_zapasow =
			sum {t in Tygodnie}
				Jedn_koszt_zapasow * Zapas[t];

	#--------------------------------------------------------------------------------------------------
	Ogr_Zapas_poczatkowy:
		Zapas[0] = 0;

	#--------------------------------------------------------------------------------------------------
	Ogr_1_3_b_Bilans_zapasow {t in Tygodnie}:
		Zapas[t-1] + Produkcja[t] = Popyt[t] + Zapas[t];

	Ogr_1_3_c_Produkcja_gdy_Gotowosc {t in Tygodnie}:
		Produkcja[t] <= Pozostaly_popyt[t] * Gotowosc[t];

	#--------------------------------------------------------------------------------------------------
	# Nierwnoci odcinajce
	#--------------------------------------------------------------------------------------------------

	NO_Okres_wyczerpania_zapasu
		{t in Tygodnie, p in {0} union Tygodnie: p <= L_tygodni-t}:
		Zapas[t-1] >=
			sum {s in t..t+p}
				Popyt[s] * (1 - sum {r in t..s} Gotowosc[r] );

#=====================================================================
	display Pozostaly_popyt;

	solve;

	printf "Zmienna;Tydzie;Warto" > Wyniki;

	printf "\n%s;;%s", 'Model',Model >> Wyniki;
	printf "\n%s;;%s", 'Dane',Dane >> Wyniki;
	printf "\n%s;;%s", 'Data', time2str(gmtime(),'%F') >> Wyniki;
	printf "\n%s;;%s", 'Godzina', time2str(gmtime(),'%H:%M') >> Wyniki;
	printf "\n%s;;%d", 'L_tygodni', L_tygodni >> Wyniki;
	printf "\n%s;;%d", 'Czas_pracy', Czas_pracy_na_tydzien >> Wyniki;

	printf "\n%s;;%3.1f", 'Calk_koszt_zapasow', Calk_koszt_zapasow >> Wyniki;
	printf "\n%s;;%3.1f", 'Calk_koszt_przezbr', Calk_koszt_przezbr >> Wyniki;
	printf "\n%s;;%3.1f", 'Calk_L_przezbr', sum {t in Tygodnie} Gotowosc[t] >> Wyniki;

	printf "\n%s;;%3.1f", 'Czas_wykonywania', Czas_wykonywania >> Wyniki;
	printf "\n%s;;%3.1f", 'Koszt_przezbr', Koszt_przezbr >> Wyniki;
	printf "\n%s;;%3.1f", 'Czas_przezbr', Czas_przezbr >> Wyniki;
	printf "\n%s;;%3.1f", 'Jedn_koszt_zapasow', Jedn_koszt_zapasow >> Wyniki;

	for {t in Tygodnie} {
		printf "\n%s;%d;%3.1f", 'Popyt', t, Popyt[t] >> Wyniki;
		printf "\n%s;%d;%3.1f", 'Produkcja', t, Produkcja[t] >> Wyniki;
		printf "\n%s;%d;%d", 'Gotowosc', t, Gotowosc[t] >> Wyniki;
	}

	for {t in Tygodnie union {0}}
		printf "\n%s;%d;%3.1f", 'Zapas', t, Zapas[t] >> Wyniki;

	#--------------------------------------------------------------------------------------------------
	printf "%s;%s;%3.1f;%3.1f;%3.1f\n", Model, Dane,
		Calk_koszt_zapasow , Calk_koszt_przezbr,
		sum {t in Tygodnie} Gotowosc[t]
		>> Podsumowanie ;

#=====================================================================
end;
